"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RESTQueryParams = exports.RESTResponseBody = exports.RESTError = exports.RESTResult = exports.RESTPage = exports.RESTRequestBody = exports.RESTPaging = exports.RESTSort = exports.SortDirection = void 0;
const common_1 = require("@nestjs/common");
const class_transformer_1 = require("class-transformer");
const class_validator_1 = require("class-validator");
var SortDirection;
(function (SortDirection) {
    SortDirection[SortDirection["asc"] = 1] = "asc";
    SortDirection[SortDirection["desc"] = -1] = "desc";
})(SortDirection || (exports.SortDirection = SortDirection = {}));
class RESTSort {
}
exports.RESTSort = RESTSort;
__decorate([
    (0, class_validator_1.IsNotEmpty)(),
    (0, class_validator_1.IsString)(),
    __metadata("design:type", String)
], RESTSort.prototype, "property", void 0);
__decorate([
    (0, class_validator_1.IsNotEmpty)(),
    (0, class_validator_1.IsNumber)(),
    (0, class_validator_1.IsEnum)(SortDirection),
    __metadata("design:type", Number)
], RESTSort.prototype, "direction", void 0);
class RESTPaging {
    extract() {
        const skip = (this.page - 1) * this.size;
        return { skip, limit: this.size };
    }
    static generate(page, size) {
        const skip = (page - 1) * size;
        return { skip, limit: size };
    }
}
exports.RESTPaging = RESTPaging;
__decorate([
    (0, class_validator_1.IsNotEmpty)(),
    (0, class_validator_1.IsInt)(),
    (0, class_transformer_1.Type)(() => Number),
    (0, class_validator_1.Min)(1),
    __metadata("design:type", Number)
], RESTPaging.prototype, "page", void 0);
__decorate([
    (0, class_validator_1.IsNotEmpty)(),
    (0, class_validator_1.IsInt)(),
    (0, class_transformer_1.Type)(() => Number),
    (0, class_validator_1.Min)(1),
    (0, class_validator_1.Max)(100),
    __metadata("design:type", Number)
], RESTPaging.prototype, "size", void 0);
class RESTRequestBody {
    isPagingPresent() {
        if (this.paging && this.paging.page) {
            return true;
        }
        else {
            return false;
        }
    }
    isSortPresent() {
        if (this.sort && this.sort.length) {
            return true;
        }
        else {
            return false;
        }
    }
    extractPaging() {
        if (this.isPagingPresent()) {
            return this.paging.extract();
        }
        return null;
    }
    extractSort() {
        const sort = {};
        if (this.isSortPresent()) {
            for (let item of this.sort) {
                sort[item.property] = item.direction;
            }
        }
        return sort;
    }
}
exports.RESTRequestBody = RESTRequestBody;
__decorate([
    (0, class_validator_1.IsNotEmpty)(),
    (0, class_validator_1.ValidateNested)(),
    __metadata("design:type", Object)
], RESTRequestBody.prototype, "data", void 0);
__decorate([
    (0, class_validator_1.IsOptional)(),
    (0, class_validator_1.ValidateNested)(),
    (0, class_transformer_1.Type)(() => RESTPaging),
    __metadata("design:type", RESTPaging)
], RESTRequestBody.prototype, "paging", void 0);
__decorate([
    (0, class_validator_1.IsOptional)(),
    (0, class_validator_1.IsArray)(),
    (0, class_validator_1.ValidateNested)(),
    (0, class_transformer_1.Type)(() => RESTSort),
    __metadata("design:type", Array)
], RESTRequestBody.prototype, "sort", void 0);
class RESTPage {
    static create(count, paging) {
        const page = new RESTPage();
        page.page = paging.page;
        page.size = paging.size;
        page.number = (count - (count % paging.size)) / paging.size;
        if (count % paging.size > 0) {
            page.number += 1;
        }
        page.total = count;
        return page;
    }
}
exports.RESTPage = RESTPage;
class RESTResult {
}
exports.RESTResult = RESTResult;
class RESTError {
    constructor(keyword, debug, statusCode) {
        this.keyword = keyword;
        this.debug = debug;
        if (!statusCode) {
            this.statusCode = common_1.HttpStatus.BAD_REQUEST;
        }
        this.statusCode = statusCode;
    }
    static dataValidation(validation) {
        const error = new RESTError('body_validation_error', null, 422);
        error.validation = validation;
        return error;
    }
    static createFromCatch(e, fallback) {
        if (e instanceof RESTError) {
            return e;
        }
        if (e.response && e.response.data && e.response.data.errors) {
            return new RESTError(e.response.data.errors[0].keyword, e.response.data.errors[0].debug, e.response.data.statusCode);
        }
        if (!fallback.statusCode) {
            fallback.statusCode = common_1.HttpStatus.BAD_REQUEST;
        }
        return new RESTError(fallback.keyword, fallback.debug, fallback.statusCode);
    }
}
exports.RESTError = RESTError;
class RESTResponseBody {
    addError(error) {
        if (!this.errors) {
            this.errors = [];
        }
        this.errors.push(error);
    }
    static success(data, page, sort) {
        const response = new RESTResponseBody();
        response.statusCode = 200;
        response.result = new RESTResult();
        response.result.data = data;
        response.result.page = page;
        response.result.sort = sort;
        return response;
    }
    static fail(errors, statusCode) {
        const response = new RESTResponseBody();
        response.statusCode = statusCode;
        response.errors = errors;
        return response;
    }
}
exports.RESTResponseBody = RESTResponseBody;
class RESTQueryParams extends RESTPaging {
    getRESTSort() {
        const result = [];
        const items = [];
        if (this.sort) {
            if (this.sort instanceof Array) {
                items.push(...this.sort);
            }
            else {
                items.push(this.sort);
            }
        }
        for (let item of items) {
            const pair = item.split(':');
            if (pair.length != 2) {
                throw new RESTError('invalid_sorting_pair', pair, 422);
            }
            if (['asc', 'desc'].indexOf(pair[1]) === -1) {
                throw new RESTError('invalid_sorting_direction', pair[1], 422);
            }
            result.push({
                property: pair[0],
                direction: pair[1] === 'asc' ? 1 : -1,
            });
        }
        return result;
    }
    extractSort() {
        const sorts = this.getRESTSort();
        const result = {};
        for (let item of sorts) {
            result[item.property] = item.direction;
        }
        return result;
    }
}
exports.RESTQueryParams = RESTQueryParams;
__decorate([
    (0, class_validator_1.IsOptional)(),
    (0, class_validator_1.IsString)({ each: true }),
    __metadata("design:type", Array)
], RESTQueryParams.prototype, "sort", void 0);
//# sourceMappingURL=rest-wrapper.class.js.map