"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SessionsService = void 0;
const axios_1 = require("@nestjs/axios");
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const rest_wrapper_class_1 = require("../models/dtos/wrappers/rest-wrapper.class");
const rxjs_1 = require("rxjs");
const mongoose_1 = require("mongoose");
const session_schema_1 = require("../models/schemas/session.schema");
const mongoose_2 = require("@nestjs/mongoose");
let SessionsService = class SessionsService {
    constructor(_sessionModel, _httpService, _configService) {
        this._sessionModel = _sessionModel;
        this._httpService = _httpService;
        this._configService = _configService;
        this._baseUrl = this._configService.getOrThrow('DL_SESSIONS__REST__BASE_URL');
    }
    async create(userId, userAgent, ip) {
        try {
            const endpoint = `${this._baseUrl}/v1/sessions/${userId}`;
            const headers = { 'user-agent': userAgent, 'x-ip': ip };
            const response$ = this._httpService.post(endpoint, null, { headers });
            const response = await (0, rxjs_1.firstValueFrom)(response$);
            if (response.data.errors) {
                throw new rest_wrapper_class_1.RESTError('sessions_api_error', response.data.errors);
            }
            if (!response.data || !response.data.result) {
                throw new rest_wrapper_class_1.RESTError('sessions_api_error', 'empty response');
            }
            return response.data.result.data;
        }
        catch (e) {
            throw rest_wrapper_class_1.RESTError.createFromCatch(e, { keyword: 'unknown', debug: e });
        }
    }
    async read(token) {
        if (!token) {
            throw new common_1.UnauthorizedException();
        }
        const session = await this._sessionModel.findOne({ token });
        if (!session) {
            throw new common_1.UnauthorizedException();
        }
        return session;
    }
};
exports.SessionsService = SessionsService;
exports.SessionsService = SessionsService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_2.InjectModel)(session_schema_1.Session.name, 'sessions-store')),
    __metadata("design:paramtypes", [mongoose_1.Model,
        axios_1.HttpService,
        config_1.ConfigService])
], SessionsService);
//# sourceMappingURL=sessions.service.js.map